'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.getRelease = getRelease;
exports.getReleaseDraft = getReleaseDraft;
exports.createRelease = createRelease;
exports.uploadAsset = uploadAsset;
exports.downloadAsset = downloadAsset;

var _log = require('./log.js');

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _requestProgress = require('request-progress');

var _requestProgress2 = _interopRequireDefault(_requestProgress);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var OWNER = 'zeit'; /* eslint-disable camelcase */

var REPO = 'pkg-fetch';
var _process$env = process.env;
var GITHUB_USERNAME = _process$env.GITHUB_USERNAME;
var GITHUB_PASSWORD = _process$env.GITHUB_PASSWORD;

var auth = { user: GITHUB_USERNAME, pass: GITHUB_PASSWORD };
var request2 = _request2.default.defaults({
  auth: auth.user ? auth : null,
  headers: { 'User-Agent': OWNER + '/' + REPO + '/' + GITHUB_USERNAME },
  timeout: 30 * 1000
});

function getRelease(tag) {
  return new _promise2.default(function (resolve, reject) {
    var url = 'https://api.github.com/repos/' + OWNER + '/' + REPO + '/releases/tags/' + tag;
    request2(url, function (error, response, body) {
      if (error) {
        _log.log.error(error.message);
        return reject((0, _log.wasReported)(error));
      }
      var release = JSON.parse(body);
      var message = release.message;

      if (message === 'Not Found') return resolve(null);
      if (message) return reject(new Error(message));
      resolve(release);
    });
  });
}

function getReleaseDraft(tag) {
  return new _promise2.default(function (resolve, reject) {
    var url = 'https://api.github.com/repos/' + OWNER + '/' + REPO + '/releases';
    request2(url, function (error, response, body) {
      if (error) {
        _log.log.error(error.message);
        return reject((0, _log.wasReported)(error));
      }
      // here we use `get release by tag` endpoint
      // because draft releases are really `untagged`.
      // seems that `get release by tag` is for non-drafts.
      // hence listing all releases and looking through them
      var releases = JSON.parse(body);
      if (releases.message) return reject(new Error(releases.message));
      var found = releases.filter(function (_ref) {
        var tag_name = _ref.tag_name;
        return tag_name === tag;
      }); // eslint-disable-line camelcase
      (0, _assert2.default)(found.length <= 1);
      if (!found.length) return resolve(null);
      resolve(found[0]);
    });
  });
}

function createRelease(tag) {
  return new _promise2.default(function (resolve, reject) {
    var form = (0, _stringify2.default)({
      tag_name: tag,
      target_commitish: 'master', // TODO maybe git rev-parse HEAD
      name: tag,
      draft: true,
      prerelease: true
    });
    var url = 'https://api.github.com/repos/' + OWNER + '/' + REPO + '/releases';
    request2.post(url, { form: form }, function (error, response, body) {
      if (error) return reject(error);
      var release = JSON.parse(body);
      if (release.message) return reject(new Error(release.message));
      resolve(release);
    });
  });
}

function uploadAsset(file, release, name) {
  (0, _assert2.default)(!/[\\/]/.test(name));
  return new _promise2.default(function (resolve, reject) {
    _fs2.default.stat(file, function (error, stat) {
      if (error) return reject(error);
      var headers = {
        'Content-Length': stat.size,
        'Content-Type': 'application/octet-stream'
      };
      var rs = _fs2.default.createReadStream(file);
      var subst = '?name=' + name;
      var url = release.upload_url.replace(/\{\?name,label\}/, subst);
      var req = request2.post(url, { headers: headers }, function (error2, response, body) {
        if (error2) return reject(error2);
        var asset = JSON.parse(body);
        var errors = asset.errors;

        if (errors && errors[0]) return reject(new Error(errors[0].code));
        if (asset.message) return reject(new Error(asset.message));
        resolve(asset);
      });
      rs.pipe(req);
    });
  });
}

function downloadAsset(asset, file, short) {
  _log.log.enableProgress(short);
  _log.log.showProgress(0);
  return new _promise2.default(function (resolve, reject) {
    var headers = { Accept: 'application/octet-stream' };
    var ws = _fs2.default.createWriteStream(file);
    var url = asset.url;
    var req = (0, _requestProgress2.default)(request2.get(url, { headers: headers }, function (error, response) {
      if (error) {
        _log.log.disableProgress();
        _log.log.error(error.message);
        return reject((0, _log.wasReported)(error));
      }
      _log.log.showProgress(100);
      _log.log.disableProgress();
      resolve(response);
    }));
    req.on('progress', function (state) {
      var p = state.percentage;
      _log.log.showProgress(p * 100);
    });
    req.pipe(ws);
  });
}